package com.bcxin.rbac.domain.repository;

import com.bcxin.rbac.domain.RbacDbReader;
import com.bcxin.rbac.domain.dtos.*;
import com.bcxin.rbac.domain.repository.tmps.CategoryResourceTmpDto;
import com.bcxin.rbac.domain.repository.tmps.ResourceTmpDto;
import com.bcxin.rbac.domain.repository.tmps.RoleResourceTmpDto;
import com.mysql.cj.util.DnsSrv;
import org.springframework.stereotype.Repository;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.persistence.EntityManager;
import javax.persistence.TypedQuery;
import java.util.*;
import java.util.stream.Collectors;

@Repository
public class RbacDbReaderImpl implements RbacDbReader {
    private final EntityManager entityManager;

    public RbacDbReaderImpl(EntityManager entityManager) {
        this.entityManager = entityManager;
    }

    @Override
    public Collection<ResourceDto> getAllTreeResources(String categoryId) {
        TypedQuery<ResourceTmpDto> resourceTmpDtoTypedQuery =
                this.entityManager.createQuery("select new com.bcxin.rbac.domain.repository.tmps.ResourceTmpDto(d.id,d.category.id,d.name,d.data,d.dataType,d.metaId,d.metaConfig,d.metaData,d.level,d.parent.id, d.creator,d.lastModifier) " +
                        " from ResourceEntity d where d.category.id=?1", ResourceTmpDto.class);

        resourceTmpDtoTypedQuery.setParameter(1, categoryId);

        Collection<ResourceTmpDto> resourceTmps = resourceTmpDtoTypedQuery.getResultList();
        if (resourceTmps.size() == 0) {
            return Collections.EMPTY_LIST;
        }

        int minLevel = resourceTmps.stream().map(ii -> ii.getLevel()).min(Integer::compare).get();
        Collection<ResourceDto> rootResources = resourceTmps.stream().filter(ii -> ii.getLevel() == minLevel)
                .map(ii -> ResourceDto.create(ii.getId(), ii.getCategoryId(), ii.getName(),
                        ii.getData(), ii.getDataType(),
                        ii.getMetaId(), ii.getMetaConfig(), ii.getMetaData(),
                        ii.getCreator(), ii.getLastModifier()))
                .collect(Collectors.toList());

        rootResources.forEach(ii -> {
            this.buildChildResources(ii, resourceTmps);
        });

        return rootResources;
    }

    @Override
    public RoleDetailDto getRoleById(String subjectId, String id) {
        TypedQuery<Map> roleQuery =
                this.entityManager.createQuery("select new map(r.id as roleId, r.name as roleName,c.id as resourceId,c.code as resourceCode, c.name as resourceName," +
                        " c.parent.id as resourceParentId,c.treePathId as resourceTreePathId,c.code as resourceCode,c.data as resourceData,c.dataType as resourceDataType,c.level as resourceLevel) " +
                        " from RoleEntity r left join r.resources c where r.subject.id=?1 and r.id=?2", Map.class);

        roleQuery.setParameter(1, subjectId);
        roleQuery.setParameter(2, id);

        Collection<Map> roleRelationData = roleQuery.getResultList();
        if (roleRelationData.size() == 0) {
            return null;
        }

        RoleDetailDto roleDetailDto = null;
        Collection<RoleResourceTmpDto> roleResourceTmpDtos = new ArrayList<>();

        for (Map map : roleRelationData) {
            if (roleDetailDto == null) {
                roleDetailDto = RoleDetailDto.create((String) map.get("roleId"), (String) map.get("roleName"));
            }
            if (map.get("resourceId") != null) {
                roleResourceTmpDtos.add(
                        RoleResourceTmpDto.create(
                                (String) map.get("resourceId"),
                                (String) map.get("resourceCode"),
                                (String) map.get("resourceName"),
                                (String) map.get("resourceData"),
                                (String) map.get("resourceDataType"),
                                (String) map.get("resourceParentId"),
                                (String) map.get("treePathId"),
                                (int) map.get("resourceLevel"),
                                (String) map.get("roleId")
                        ));
            }
        }

        Collection<RoleResourceTmpDto> rootResourceTmps = roleResourceTmpDtos.stream().filter(ii ->
                        !roleResourceTmpDtos.stream().anyMatch(ix -> ix.getParentId().equals(ii.getId())))
                .collect(Collectors.toList());

        for (RoleResourceTmpDto rt : rootResourceTmps) {
            RoleDetailDto.RoleSelectedResourceDto newRoleDetailDto =
                    RoleDetailDto.RoleSelectedResourceDto.create(
                            rt.getId(), rt.getCode(),
                            rt.getName(), rt.getTreePathId());
            roleDetailDto.addSelectedResource(newRoleDetailDto);

            buildRoleResource(newRoleDetailDto, roleResourceTmpDtos);
        }

        return roleDetailDto;
    }

    @Override
    public Collection<RoleDto> getAllRoles(String subjectId) {
        TypedQuery<RoleDto> roleDtoTypedQuery =
                this.entityManager.createQuery("select new com.bcxin.rbac.domain.dtos.RoleDto(d.id,d.name) " +
                        " from RoleEntity d where d.subject.id=?1", RoleDto.class);
        roleDtoTypedQuery.setParameter(1, subjectId);

        return roleDtoTypedQuery.getResultList();
    }

    @Override
    public UserDetailDto getUserDetailById(String subjectId,String id) {
        TypedQuery<Map> roleQuery =
                this.entityManager.createQuery("select new map(u.id as userId, u.name as userName, u.referencedId as uReferencedId, r.id as roleId, r.name as roleName,c.id as resourceId,c.code as resourceCode, c.name as resourceName," +
                        " c.parent.id as resourceParentId,c.treePathId as resourceTreePathId,c.code as resourceCode,c.data as resourceData,c.dataType as resourceDataType,c.level as resourceLevel) " +
                        " from UserEntity u left join u.roles r left join r.resources c where u.subject.id=?1 and u.id=?2", Map.class);
        roleQuery.setParameter(2, id);
        roleQuery.setParameter(1, subjectId);

        Collection<Map> roleRelationData = roleQuery.getResultList();
        if (roleRelationData.size() == 0) {
            return null;
        }

        UserDetailDto userDetailDto = null;
        Collection<RoleResourceTmpDto> roleResourceTmpDtos = new ArrayList<>();
        for (Map map : roleRelationData) {
            if (userDetailDto == null) {
                userDetailDto = UserDetailDto.create(
                        (String) map.get("userId"),
                        (String) map.get("userName"),
                        (String) map.get("uReferencedId")
                );
            }
            if (map.get("roleId") != null) {
                String roleId = (String) map.get("roleId");
                Optional<RoleDetailDto> roleDetailDtoOptional = userDetailDto.getRoleDetails().stream()
                        .filter(ii -> ii.getId().equals(roleId))
                        .findFirst();
                RoleDetailDto roleDetailDto = null;
                if (roleDetailDtoOptional.isPresent()) {
                    roleDetailDto = roleDetailDtoOptional.get();
                }

                if (roleDetailDto == null) {
                    roleDetailDto = RoleDetailDto.create((String) map.get("roleId"), (String) map.get("roleName"));
                    userDetailDto.assignRoleDetail(roleDetailDto);
                }

                /**
                 * 所有的其他数据
                 */
                if (map.get("resourceId") != null) {
                    roleResourceTmpDtos.add(
                            RoleResourceTmpDto.create(
                                    (String) map.get("resourceId"),
                                    (String) map.get("resourceCode"),
                                    (String) map.get("resourceName"),
                                    (String) map.get("resourceData"),
                                    (String) map.get("resourceDataType"),
                                    (String) map.get("resourceParentId"),
                                    (String) map.get("treePathId"),
                                    (int) map.get("resourceLevel"),
                                    roleId
                            ));
                }
            }
        }

        for (RoleDetailDto selectedRoleDetail : userDetailDto.getRoleDetails()) {
            Collection<RoleResourceTmpDto> rootResourceTmps = roleResourceTmpDtos.stream().filter(ii ->
                            !roleResourceTmpDtos.stream().anyMatch(ix -> selectedRoleDetail.getId().equals(ix.getRoleId()) &&
                                    ix.getParentId().equals(ii.getId())))
                    .collect(Collectors.toList());

            for (RoleResourceTmpDto rt : rootResourceTmps) {
                RoleDetailDto.RoleSelectedResourceDto newRoleDetailDto =
                        RoleDetailDto.RoleSelectedResourceDto.create(
                                rt.getId(), rt.getCode(),
                                rt.getName(), rt.getTreePathId());
                selectedRoleDetail.addSelectedResource(newRoleDetailDto);

                buildRoleResource(newRoleDetailDto, roleResourceTmpDtos);
            }
        }

        return userDetailDto;
    }

    @Override
    public Collection<CategoryTreeDto> getTreeCategories(Collection<String> categoryIds) {
        return getCategoryTreeDtosByIds(categoryIds);
    }

    @Override
    public Collection<CategoryTreeDto> getAllTreeCategories() {
        return getCategoryTreeDtosByIds(null);
    }

    private void buildChildResources(ResourceDto parent, Collection<ResourceTmpDto> allResources) {
        Collection<ResourceDto> selectedResources = allResources.stream().filter(ii -> StringUtils.hasLength(ii.getParentId()) && ii.getParentId().equals(parent.getId()))
                .map(ii -> ResourceDto.create(ii.getId(), ii.getCategoryId(), ii.getName(), ii.getData(), ii.getDataType(),
                        ii.getMetaId(), ii.getMetaConfig(), ii.getMetaData(),
                        ii.getCreator(), ii.getLastModifier()))
                .collect(Collectors.toList());
        for (ResourceDto resourceDto : selectedResources) {
            this.buildChildResources(resourceDto, allResources);

            parent.addChildResource(resourceDto);
        }
    }

    private void buildRoleResource(RoleDetailDto.RoleSelectedResourceDto roleSelectedResourceDto,
                                   Collection<RoleResourceTmpDto> allResourceTmpDtos) {
        Collection<RoleResourceTmpDto> selectedResourceTmps = allResourceTmpDtos
                .stream().filter(ii -> StringUtils.hasLength(ii.getParentId()) && ii.getParentId().equals(roleSelectedResourceDto.getId()))
                .collect(Collectors.toList());

        for (RoleResourceTmpDto rt : selectedResourceTmps) {
            RoleDetailDto.RoleSelectedResourceDto newRoleDetailDto = RoleDetailDto.RoleSelectedResourceDto.create(
                    rt.getId(), rt.getCode(),
                    rt.getName(), rt.getTreePathId());

            roleSelectedResourceDto.addChildResource(newRoleDetailDto);

            this.buildRoleResource(newRoleDetailDto, allResourceTmpDtos);
        }
    }

    public Collection<CategoryTreeDto> getCategoryTreeDtosByIds(Collection<String> categoryIds) {
        String qString="select new com.bcxin.rbac.domain.repository.tmps.CategoryResourceTmpDto(c.id,c.name,d.id,d.name,d.data,d.dataType,d.metaId,d.metaConfig,d.metaData,d.level,d.parent.id, d.creator,d.lastModifier) " +
                " from ResourceEntity d join d.category c ";

        Map<Integer,Object> parameters = new HashMap<>();
        if(!CollectionUtils.isEmpty(categoryIds)) {
            qString = "select new com.bcxin.rbac.domain.repository.tmps.CategoryResourceTmpDto(c.id,c.name,d.id,d.name,d.data,d.dataType,d.metaId,d.metaConfig,d.metaData,d.level,d.parent.id, d.creator,d.lastModifier) " +
                    " from ResourceEntity d join d.category c where c.id in (?1)";
            parameters.put(1, categoryIds);
        }
        TypedQuery<CategoryResourceTmpDto> categoryResourceTmpDtoTypedQuery =
                this.entityManager.createQuery(qString, CategoryResourceTmpDto.class);

        for(Integer key:parameters.keySet()) {
            categoryResourceTmpDtoTypedQuery.setParameter(key, parameters.get(key));
        }

        Collection<CategoryResourceTmpDto> categoryResourceTmps = categoryResourceTmpDtoTypedQuery.getResultList();
        if (categoryResourceTmps.size() == 0) {
            return Collections.EMPTY_LIST;
        }
        Collection<CategoryTreeDto> categoryTreeDtos = new ArrayList<>();
        for (CategoryResourceTmpDto crdt : categoryResourceTmps) {
            Optional<CategoryTreeDto> selectedCategoryOptional = categoryTreeDtos.stream().filter(ii -> ii.getId().equals(crdt.getCategoryId())).findFirst();
            CategoryTreeDto selectedCategory = null;
            if (!selectedCategoryOptional.isPresent()) {
                selectedCategory = new CategoryTreeDto(crdt.getCategoryId(), crdt.getCategoryName(), crdt.getCategoryName());
                categoryTreeDtos.add(selectedCategory);
            }
        }

        categoryTreeDtos.forEach(ctd -> {
            Collection<ResourceTmpDto> resourceTmps =
                    categoryResourceTmps.stream().filter(ii -> ii.getCategoryId().equals(ctd.getId()))
                            .collect(Collectors.toList());

            int minLevel = resourceTmps.stream().map(ii -> ii.getLevel()).min(Integer::compare).get();
            Collection<ResourceDto> rootResources = resourceTmps.stream().filter(ii -> ii.getLevel() == minLevel)
                    .map(ii -> ResourceDto.create(ii.getId(), ii.getCategoryId(), ii.getName(),
                            ii.getData(), ii.getDataType(),
                            ii.getMetaId(), ii.getMetaConfig(), ii.getMetaData(),
                            ii.getCreator(), ii.getLastModifier()))
                    .collect(Collectors.toList());

            rootResources.forEach(ii -> {
                this.buildChildResources(ii, resourceTmps);
            });
            ctd.assignResources(rootResources);
        });

        return categoryTreeDtos;
    }
}
